﻿Type=StaticCode
Version=1.60
@EndOfDesignText@
' ************************************************************************
' * Game world canvas "class"                                            *
' * Version: 1.0.0.0 beta                                                *
' * (C)Copyright 2011 M. le Roux                                         *
' ************************************************************************

'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

	Type GWCanvas (ParentView As View, ParentCanvas As Canvas, BGColor As Int, Actors As List, BGImage As Bitmap, BGImageRect As Rect)
	
	Dim Radians As Float : Radians = cPI / 180
End Sub

' *** Constructor ...
Sub Create (ParentView As View) As GWCanvas
	Dim res As GWCanvas
	res.Initialize
	res.Actors.Initialize
	res.ParentView = ParentView
	res.BGColor = Colors.Black ' *** Default background color ...
	' *** Initialize canvas (would need to do this again if you need to change the parent view) ...
	res.ParentCanvas.Initialize(res.ParentView)
	' *** Return "object" ...
	Return res
End Sub

#Region Methods ...

' *** Add an actor "instance" to the world ...
Sub AddActor (Instance As GWCanvas, ActorToAdd As Object) As Object
	Instance.Actors.Add(ActorToAdd)
	Return ActorToAdd
End Sub

' *** Returns the name (basic) of a given type ...
Sub TypeName (o As Object) As String
	Dim s As String : s = GetType(o)
	Return s.SubString (s.IndexOf("$_") + 2)
End Sub

' *** Render the game world ...
Sub Render (Instance As GWCanvas)
	' *** Draw background image (or part of image) if initialized, else draw background color if not Colors.Transparent ...
	If Instance.BGImage.IsInitialized Then
		Dim destRect As Rect
		destRect.Initialize(0, 0, Instance.ParentView.Width, Instance.ParentView.Height)
		If Instance.BGImageRect.IsInitialized Then 
			Instance.ParentCanvas.DrawBitmap(Instance.BGImage, Instance.BGImageRect, destRect) ' *** Image rect defined, draw partial bitmap ...
		Else
			Instance.ParentCanvas.DrawBitmap(Instance.BGImage, Null, destRect) ' *** No source rect defined, draw full bitmap ...
		End If
	Else If Instance.BGColor <> Colors.Transparent Then 
		Instance.ParentCanvas.DrawColor(Instance.BGColor)
	End If
	' *** Draw actors ...
	Dim o As Object
	For c = 0 To Instance.Actors.Size-1
		o = Instance.Actors.Get(c)
		Select TypeName (o)
			Case "gwsprite": 
				Dim spr As GWSprite : spr = o
				If spr.Base.Visible Then GWSprite.Render(o, Instance.ParentCanvas)
		End Select
	Next
	' *** Invalidate parent view ...
	Instance.ParentView.Invalidate
End Sub

#End Region
