﻿Type=StaticCode
Version=1.60
@EndOfDesignText@
' ************************************************************************
' * Game world sprite "class"                                            *
' * Version: 1.0.0.0 beta                                                *
' * (C)Copyright 2011 M. le Roux                                         *
' ************************************************************************

'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

	' *** We include "base" here as the base class (inherited from - this would enable us to "inherit" from multiple classes ;-)
	Type GWSprite (Base As GWActor, Rotation As Float, Img As Bitmap, FrameWidth, FrameIndex As Int, DrawWidth, DrawHeight As Int)
End Sub

' *** Constructor ...
' *** FrameWidth must be -1 if only 1 frame present ...
Sub Create (Name As String, x, y As Int, Img As Bitmap, FrameWidth As Int) As GWSprite
	Dim r As GWSprite
	r.Initialize
	r.Base = GWActor.Create (Name, x, y)
	r.Img = img
	r.FrameWidth = FrameWidth
	r.FrameIndex = 0
	If FrameWidth = -1 Then
		r.DrawWidth = img.Width
	Else
		r.DrawWidth = FrameWidth
	End If
	r.DrawHeight = img.Height
	Return r
End Sub

' *** "Overloaded" Constructor ...
Sub Create2 (Name As String, x, y As Int, Img As Bitmap) As GWSprite
	Return Create(Name, x, y, Img, -1)
End Sub

#Region Methods ...

' *** Render this sprite on the given canvas ...
Sub Render (Instance As GWSprite, canv As Canvas)
	If Instance.Base.Visible Then
		Dim srcRect, destRect As Rect
		If Instance.FrameWidth = -1 Then
			destRect.Initialize(Instance.Base.X, Instance.Base.Y, Instance.Base.X + Instance.DrawWidth, Instance.Base.Y + Instance.DrawHeight)
		Else
			Dim x As Int
			x = Instance.FrameIndex * Instance.FrameWidth
			srcRect.Initialize(x, 0, x + Instance.FrameWidth, Instance.Img.Height)
			destRect.Initialize(Instance.Base.X, Instance.Base.Y, Instance.Base.X + Instance.DrawWidth, Instance.Base.Y + Instance.DrawHeight)
		End If
		' *** Check rotation ...
		If Instance.Rotation > 360 Then Instance.Rotation = 0
		If Instance.Rotation < 0 Then Instance.Rotation = 360		
		' *** Render sprite ...
		If srcRect.IsInitialized Then
			canv.DrawBitmapRotated(Instance.Img, srcRect, destRect, Instance.Rotation)
		Else
			canv.DrawBitmapRotated(Instance.Img, Null, destRect, Instance.Rotation)
		End If
	End If
End Sub

' *** Calculates the number of frames for this sprite (uses images width / FrameWidth) ...
Sub MaxFrames (Instance As GWSprite) As Int
	If Instance.FrameWidth > -1 Then
		Return Instance.Img.Width / Instance.FrameWidth
	Else
		Return 1
	End If
End Sub

' *** Next frame ...
Sub NextFrame (Instance As GWSprite)
	Instance.FrameIndex = Instance.FrameIndex + 1
	If Instance.FrameIndex >= MaxFrames (Instance) Then Instance.FrameIndex = 0
End Sub

' *** Previous frame ...
Sub PreviousFrame (Instance As GWSprite)
	Instance.FrameIndex = Instance.FrameIndex - 1
	If Instance.FrameIndex < 0 Then Instance.FrameIndex = MaxFrames (Instance) - 1
End Sub

' *** Do some simple collision detection ...
' *** Need more advanced methods!!!!
Sub TestCollide (Instance1 As GWSprite, Instance2 As GWSprite) As Boolean
	Return ((Instance1.Base.X >= Instance2.Base.X) AND (Instance1.Base.X <= Instance2.Base.X + Get_Width (Instance2)) AND _
	        (Instance1.Base.Y >= Instance2.Base.Y) AND (Instance1.Base.Y <= Instance2.Base.Y + Get_Height (Instance2))) OR _
		   ((Instance2.Base.X >= Instance1.Base.X) AND (Instance2.Base.X <= Instance1.Base.X + Get_Width (Instance1)) AND _
	        (Instance2.Base.Y >= Instance1.Base.Y) AND (Instance2.Base.Y <= Instance1.Base.Y + Get_Height (Instance1)))
End Sub

' *** Move sprite by Rotation andgle and specified speed (negative speed to move backward) ...
' *** We can probably optimize this quite a lot but for now it works ...
Sub MoveByAngle (Instance As GWSprite, Speed As Int)
	Dim r As Float : r = Instance.Rotation * GWCanvas.Radians	
	Instance.Base.X = Instance.Base.X + Cos(r) * Speed
	Instance.Base.Y = Instance.Base.Y + Sin(r) * Speed
End Sub

#End Region

#Region Properties

' *** Returns sprite width ...
Sub Get_Width (Instance As GWSprite) As Int
	Return Instance.DrawWidth
End Sub

' *** Returns sprite height ...
Sub Get_Height (Instance As GWSprite) As Int
	Return Instance.DrawHeight
End Sub

#End Region